@@ -6,6 +6,8 @@ Python client for InfluxDB
 from functools import wraps
 import json
 import socket
+import time
+import threading
 import random
 import requests
 import requests.exceptions
@@ -55,6 +57,8 @@ class InfluxDBClient(object):
     :type use_udp: int
     :param udp_port: UDP port to connect to InfluxDB, defaults to 4444
     :type udp_port: int
+    :param proxies: HTTP(S) proxy to use for Requests, defaults to {}
+    :type proxies: dict
     """
 
     def __init__(self,
@@ -68,9 +72,10 @@ class InfluxDBClient(object):
                  timeout=None,
                  use_udp=False,
                  udp_port=4444,
+                 proxies=None,
                  ):
         """Construct a new InfluxDBClient object."""
-        self._host = host
+        self.__host = host
         self._port = port
         self._username = username
         self._password = password
@@ -90,7 +95,12 @@ class InfluxDBClient(object):
         if ssl is True:
             self._scheme = "https"
 
-        self._baseurl = "{0}://{1}:{2}".format(
+        if proxies is None:
+            self._proxies = {}
+        else:
+            self._proxies = proxies
+
+        self.__baseurl = "{0}://{1}:{2}".format(
             self._scheme,
             self._host,
             self._port)
@@ -100,6 +110,22 @@ class InfluxDBClient(object):
             'Accept': 'text/plain'
         }
 
+    # _baseurl and _host are properties to allow InfluxDBClusterClient
+    # to override them with thread-local variables
+    @property
+    def _baseurl(self):
+        return self._get_baseurl()
+
+    def _get_baseurl(self):
+        return self.__baseurl
+
+    @property
+    def _host(self):
+        return self._get_host()
+
+    def _get_host(self):
+        return self.__host
+
     @staticmethod
     def from_DSN(dsn, **kwargs):
         """Return an instance of :class:`~.InfluxDBClient` from the provided
@@ -132,36 +158,10 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             additional `udp_port` parameter (cf. examples).
         """
 
-        init_args = {}
-        conn_params = urlparse(dsn)
-        scheme_info = conn_params.scheme.split('+')
-        if len(scheme_info) == 1:
-            scheme = scheme_info[0]
-            modifier = None
-        else:
-            modifier, scheme = scheme_info
-
-        if scheme != 'influxdb':
-            raise ValueError('Unknown scheme "{}".'.format(scheme))
-        if modifier:
-            if modifier == 'udp':
-                init_args['use_udp'] = True
-            elif modifier == 'https':
-                init_args['ssl'] = True
-            else:
-                raise ValueError('Unknown modifier "{}".'.format(modifier))
-
-        if conn_params.hostname:
-            init_args['host'] = conn_params.hostname
-        if conn_params.port:
-            init_args['port'] = conn_params.port
-        if conn_params.username:
-            init_args['username'] = conn_params.username
-        if conn_params.password:
-            init_args['password'] = conn_params.password
-        if conn_params.path and len(conn_params.path) > 1:
-            init_args['database'] = conn_params.path[1:]
-
+        init_args = parse_dsn(dsn)
+        host, port = init_args.pop('hosts')[0]
+        init_args['host'] = host
+        init_args['port'] = port
         init_args.update(kwargs)
 
         return InfluxDBClient(**init_args)
@@ -202,8 +202,10 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :type expected_response_code: int
         :returns: the response from the request
         :rtype: :class:`requests.Response`
+        :raises InfluxDBServerError: if the response code is any server error
+            code (5xx)
         :raises InfluxDBClientError: if the response code is not the
-            same as `expected_response_code`
+            same as `expected_response_code` and is not a server error code
         """
         url = "{0}/{1}".format(self._baseurl, url)
 
@@ -227,6 +229,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
                     params=params,
                     data=data,
                     headers=headers,
+                    proxies=self._proxies,
                     verify=self._verify_ssl,
                     timeout=self._timeout
                 )
@@ -237,7 +240,9 @@ localhost:8086/databasename', timeout=5, udp_port=159)
                 else:
                     raise e
 
-        if response.status_code == expected_response_code:
+        if response.status_code >= 500 and response.status_code < 600:
+            raise InfluxDBServerError(response.content)
+        elif response.status_code == expected_response_code:
             return response
         else:
             raise InfluxDBClientError(response.content, response.status_code)
@@ -276,7 +281,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
 
     def query(self,
               query,
-              params={},
+              params=None,
               epoch=None,
               expected_response_code=200,
               database=None,
@@ -303,6 +308,9 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :returns: the queried data
         :rtype: :class:`~.ResultSet`
         """
+        if params is None:
+            params = {}
+
         params['q'] = query
         params['db'] = database or self._database
 
@@ -446,13 +454,16 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         """
         return list(self.query("SHOW DATABASES").get_points())
 
-    def create_database(self, dbname):
+    def create_database(self, dbname, if_not_exists=False):
         """Create a new database in InfluxDB.
 
         :param dbname: the name of the database to create
         :type dbname: str
         """
-        self.query("CREATE DATABASE %s" % dbname)
+        if if_not_exists:
+            self.query("CREATE DATABASE IF NOT EXISTS \"%s\"" % dbname)
+        else:
+            self.query("CREATE DATABASE \"%s\"" % dbname)
 
     def drop_database(self, dbname):
         """Drop a database from InfluxDB.
@@ -460,7 +471,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param dbname: the name of the database to drop
         :type dbname: str
         """
-        self.query("DROP DATABASE %s" % dbname)
+        self.query("DROP DATABASE \"%s\"" % dbname)
 
     def create_retention_policy(self, name, duration, replication,
                                 database=None, default=False):
@@ -519,12 +530,12 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             should be set. Otherwise the operation will fail.
         """
         query_string = (
-            "ALTER RETENTION POLICY {} ON {}"
+            "ALTER RETENTION POLICY {0} ON {1}"
         ).format(name, database or self._database)
         if duration:
-            query_string += " DURATION {}".format(duration)
+            query_string += " DURATION {0}".format(duration)
         if replication:
-            query_string += " REPLICATION {}".format(replication)
+            query_string += " REPLICATION {0}".format(replication)
         if default is True:
             query_string += " DEFAULT"
 
@@ -584,6 +595,25 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             )
         return series
 
+    def get_list_servers(self):
+        """Get the list of servers in InfluxDB cluster.
+
+        :returns: all nodes in InfluxDB cluster
+        :rtype: list of dictionaries
+
+        :Example:
+
+        ::
+
+            >> servers = client.get_list_servers()
+            >> servers
+            [{'cluster_addr': 'server01:8088',
+              'id': 1,
+              'raft': True,
+              'raft-leader': True}]
+        """
+        return list(self.query("SHOW SERVERS").get_points())
+
     def get_list_users(self):
         """Get the list of all users in InfluxDB.
 
@@ -613,7 +643,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             privileges or not
         :type admin: boolean
         """
-        text = "CREATE USER {} WITH PASSWORD '{}'".format(username, password)
+        text = "CREATE USER {0} WITH PASSWORD '{1}'".format(username, password)
         if admin:
             text += ' WITH ALL PRIVILEGES'
         self.query(text)
@@ -624,7 +654,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param username: the username to drop
         :type username: str
         """
-        text = "DROP USER {}".format(username)
+        text = "DROP USER {0}".format(username)
         self.query(text)
 
     def set_user_password(self, username, password):
@@ -635,7 +665,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param password: the new password for the user
         :type password: str
         """
-        text = "SET PASSWORD FOR {} = '{}'".format(username, password)
+        text = "SET PASSWORD FOR {0} = '{1}'".format(username, password)
         self.query(text)
 
     def delete_series(self, database=None, measurement=None, tags=None):
@@ -653,10 +683,10 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         database = database or self._database
         query_str = 'DROP SERIES'
         if measurement:
-            query_str += ' FROM "{}"'.format(measurement)
+            query_str += ' FROM "{0}"'.format(measurement)
 
         if tags:
-            query_str += ' WHERE ' + ' and '.join(["{}='{}'".format(k, v)
+            query_str += ' WHERE ' + ' and '.join(["{0}='{1}'".format(k, v)
                                                    for k, v in tags.items()])
         self.query(query_str, database=database)
 
@@ -669,7 +699,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         .. note:: Only a cluster administrator can create/ drop databases
             and manage users.
         """
-        text = "REVOKE ALL PRIVILEGES FROM {}".format(username)
+        text = "REVOKE ALL PRIVILEGES FROM {0}".format(username)
         self.query(text)
 
     def grant_privilege(self, privilege, database, username):
@@ -683,9 +713,9 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param username: the username to grant the privilege to
         :type username: str
         """
-        text = "GRANT {} ON {} TO {}".format(privilege,
-                                             database,
-                                             username)
+        text = "GRANT {0} ON {1} TO {2}".format(privilege,
+                                                database,
+                                                username)
         self.query(text)
 
     def revoke_privilege(self, privilege, database, username):
@@ -699,9 +729,9 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param username: the username to revoke the privilege from
         :type username: str
         """
-        text = "REVOKE {} ON {} FROM {}".format(privilege,
-                                                database,
-                                                username)
+        text = "REVOKE {0} ON {1} FROM {2}".format(privilege,
+                                                   database,
+                                                   username)
         self.query(text)
 
     def send_packet(self, packet):
@@ -716,8 +746,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
 
 class InfluxDBClusterClient(object):
     """The :class:`~.InfluxDBClusterClient` is the client for connecting
-    to a cluster of InfluxDB servers. It's basically a proxy to multiple
-    InfluxDBClients.
+    to a cluster of InfluxDB servers. Each query hits different host from the
+    list of hosts.
 
     :param hosts: all hosts to be included in the cluster, each of which
         should be in the format (address, port),
@@ -727,9 +757,12 @@ class InfluxDBClusterClient(object):
     :param shuffle: whether the queries should hit servers evenly(randomly),
         defaults to True
     :type shuffle: bool
-    :param client_base_class: the base class for all clients in the cluster.
+    :param client_base_class: the base class for the cluster client.
         This parameter is used to enable the support of different client
         types. Defaults to :class:`~.InfluxDBClient`
+    :param healing_delay: the delay in seconds, counting from last failure of
+        a server, before re-adding server to the list of working servers.
+        Defaults to 15 minutes (900 seconds)
     """
 
     def __init__(self,
@@ -744,27 +777,37 @@ class InfluxDBClusterClient(object):
                  udp_port=4444,
                  shuffle=True,
                  client_base_class=InfluxDBClient,
+                 healing_delay=900,
                  ):
-        self.clients = []
-        self.bad_clients = []   # Corresponding server has failures in history
+        self.clients = [self]  # Keep it backwards compatible
+        self.hosts = hosts
+        self.bad_hosts = []   # Corresponding server has failures in history
         self.shuffle = shuffle
-        for h in hosts:
-            self.clients.append(client_base_class(host=h[0], port=h[1],
-                                                  username=username,
-                                                  password=password,
-                                                  database=database,
-                                                  ssl=ssl,
-                                                  verify_ssl=verify_ssl,
-                                                  timeout=timeout,
-                                                  use_udp=use_udp,
-                                                  udp_port=udp_port))
+        self.healing_delay = healing_delay
+        self._last_healing = time.time()
+        host, port = self.hosts[0]
+        self._hosts_lock = threading.Lock()
+        self._thread_local = threading.local()
+        self._client = client_base_class(host=host,
+                                         port=port,
+                                         username=username,
+                                         password=password,
+                                         database=database,
+                                         ssl=ssl,
+                                         verify_ssl=verify_ssl,
+                                         timeout=timeout,
+                                         use_udp=use_udp,
+                                         udp_port=udp_port)
         for method in dir(client_base_class):
-            if method.startswith('_'):
+            orig_attr = getattr(client_base_class, method, '')
+            if method.startswith('_') or not callable(orig_attr):
                 continue
-            orig_func = getattr(client_base_class, method)
-            if not callable(orig_func):
-                continue
-            setattr(self, method, self._make_func(orig_func))
+
+            setattr(self, method, self._make_func(orig_attr))
+
+        self._client._get_host = self._get_host
+        self._client._get_baseurl = self._get_baseurl
+        self._update_client_host(self.hosts[0])
 
     @staticmethod
     def from_DSN(dsn, client_base_class=InfluxDBClient,
@@ -787,53 +830,115 @@ class InfluxDBClusterClient(object):
 @host1:8086,usr:pwd@host2:8086/db_name', timeout=5)
             >> type(cluster)
             <class 'influxdb.client.InfluxDBClusterClient'>
-            >> cluster.clients
-            [<influxdb.client.InfluxDBClient at 0x7feb480295d0>,
+            >> cluster.hosts
+            [('host1', 8086), ('host2', 8086)]
+            >> cluster._client
              <influxdb.client.InfluxDBClient at 0x7feb438ec950>]
         """
-        conn_params = urlparse(dsn)
-        netlocs = conn_params.netloc.split(',')
-        cluster_client = InfluxDBClusterClient(
-            hosts=[],
-            client_base_class=client_base_class,
-            shuffle=shuffle,
-            **kwargs)
-        for netloc in netlocs:
-            single_dsn = '%(scheme)s://%(netloc)s%(path)s' % (
-                {'scheme': conn_params.scheme,
-                 'netloc': netloc,
-                 'path': conn_params.path}
-            )
-            cluster_client.clients.append(client_base_class.from_DSN(
-                single_dsn,
-                **kwargs))
+        init_args = parse_dsn(dsn)
+        init_args.update(**kwargs)
+        init_args['shuffle'] = shuffle
+        init_args['client_base_class'] = client_base_class
+        cluster_client = InfluxDBClusterClient(**init_args)
         return cluster_client
 
+    def _update_client_host(self, host):
+        self._thread_local.host, self._thread_local.port = host
+        self._thread_local.baseurl = "{0}://{1}:{2}".format(
+            self._client._scheme,
+            self._client._host,
+            self._client._port
+        )
+
+    def _get_baseurl(self):
+        return self._thread_local.baseurl
+
+    def _get_host(self):
+        return self._thread_local.host
+
     def _make_func(self, orig_func):
 
         @wraps(orig_func)
         def func(*args, **kwargs):
-            if self.shuffle:
-                random.shuffle(self.clients)
-            clients = self.clients + self.bad_clients
-            for c in clients:
-                bad_client = False
+            now = time.time()
+            with self._hosts_lock:
+                if (self.bad_hosts and
+                        self._last_healing + self.healing_delay < now):
+                    h = self.bad_hosts.pop(0)
+                    self.hosts.append(h)
+                    self._last_healing = now
+
+                if self.shuffle:
+                    random.shuffle(self.hosts)
+
+                hosts = self.hosts + self.bad_hosts
+
+            for h in hosts:
+                bad_host = False
                 try:
-                    return orig_func(c, *args, **kwargs)
+                    self._update_client_host(h)
+                    return orig_func(self._client, *args, **kwargs)
                 except InfluxDBClientError as e:
                     # Errors caused by user's requests, re-raise
                     raise e
                 except Exception as e:
                     # Errors that might caused by server failure, try another
-                    bad_client = True
-                    if c in self.clients:
-                        self.clients.remove(c)
-                        self.bad_clients.append(c)
+                    bad_host = True
+                    with self._hosts_lock:
+                        if h in self.hosts:
+                            self.hosts.remove(h)
+                            self.bad_hosts.append(h)
+                        self._last_healing = now
                 finally:
-                    if not bad_client and c in self.bad_clients:
-                        self.bad_clients.remove(c)
-                        self.clients.append(c)
+                    with self._hosts_lock:
+                        if not bad_host and h in self.bad_hosts:
+                            self.bad_hosts.remove(h)
+                            self.hosts.append(h)
 
             raise InfluxDBServerError("InfluxDB: no viable server!")
 
         return func
+
+
+def parse_dsn(dsn):
+    conn_params = urlparse(dsn)
+    init_args = {}
+    scheme_info = conn_params.scheme.split('+')
+    if len(scheme_info) == 1:
+        scheme = scheme_info[0]
+        modifier = None
+    else:
+        modifier, scheme = scheme_info
+
+    if scheme != 'influxdb':
+        raise ValueError('Unknown scheme "{0}".'.format(scheme))
+
+    if modifier:
+        if modifier == 'udp':
+            init_args['use_udp'] = True
+        elif modifier == 'https':
+            init_args['ssl'] = True
+        else:
+            raise ValueError('Unknown modifier "{0}".'.format(modifier))
+
+    netlocs = conn_params.netloc.split(',')
+
+    init_args['hosts'] = []
+    for netloc in netlocs:
+        parsed = _parse_netloc(netloc)
+        init_args['hosts'].append((parsed['host'], int(parsed['port'])))
+        init_args['username'] = parsed['username']
+        init_args['password'] = parsed['password']
+
+    if conn_params.path and len(conn_params.path) > 1:
+        init_args['database'] = conn_params.path[1:]
+
+    return init_args
+
+
+def _parse_netloc(netloc):
+    info = urlparse("http://{0}".format(netloc))
+    return {'username': info.username or None,
+            'password': info.password or None,
+            'host': info.hostname or 'localhost',
+            'port': info.port or 8086}
