@@ -1,7 +1,11 @@
 # -*- coding: utf-8 -*-
-import unittest
+
 import sys
 import os
+if sys.version_info < (2, 7):
+    import unittest2 as unittest
+else:
+    import unittest
 
 using_pypy = hasattr(sys, "pypy_version_info")
 skipIfPYpy = unittest.skipIf(using_pypy, "Skipping this test on pypy.")
