@@ -18,13 +18,18 @@ import json
 import requests
 import requests.exceptions
 import socket
-import unittest
+import time
 import requests_mock
 import random
 from nose.tools import raises
 from mock import patch
 import warnings
 import mock
+import sys
+if sys.version_info < (2, 7):
+    import unittest2 as unittest
+else:
+    import unittest
 
 from influxdb import InfluxDBClient, InfluxDBClusterClient
 from influxdb.client import InfluxDBServerError
@@ -95,7 +100,7 @@ class TestInfluxDBClient(unittest.TestCase):
             }
         ]
 
-        self.dsn_string = 'influxdb://uSr:pWd@host:1886/db'
+        self.dsn_string = 'influxdb://uSr:pWd@my.host.fr:1886/db'
 
     def test_scheme(self):
         cli = InfluxDBClient('host', 8086, 'username', 'password', 'database')
@@ -107,8 +112,11 @@ class TestInfluxDBClient(unittest.TestCase):
         self.assertEqual('https://host:8086', cli._baseurl)
 
     def test_dsn(self):
+        cli = InfluxDBClient.from_DSN('influxdb://192.168.0.1:1886')
+        self.assertEqual('http://192.168.0.1:1886', cli._baseurl)
+
         cli = InfluxDBClient.from_DSN(self.dsn_string)
-        self.assertEqual('http://host:1886', cli._baseurl)
+        self.assertEqual('http://my.host.fr:1886', cli._baseurl)
         self.assertEqual('uSr', cli._username)
         self.assertEqual('pWd', cli._password)
         self.assertEqual('db', cli._database)
@@ -118,11 +126,11 @@ class TestInfluxDBClient(unittest.TestCase):
         self.assertTrue(cli.use_udp)
 
         cli = InfluxDBClient.from_DSN('https+' + self.dsn_string)
-        self.assertEqual('https://host:1886', cli._baseurl)
+        self.assertEqual('https://my.host.fr:1886', cli._baseurl)
 
         cli = InfluxDBClient.from_DSN('https+' + self.dsn_string,
                                       **{'ssl': False})
-        self.assertEqual('http://host:1886', cli._baseurl)
+        self.assertEqual('http://my.host.fr:1886', cli._baseurl)
 
     def test_switch_database(self):
         cli = InfluxDBClient('host', 8086, 'username', 'password', 'database')
@@ -337,12 +345,13 @@ class TestInfluxDBClient(unittest.TestCase):
             cli.write_points_with_precision([])
 
     def test_query(self):
-        example_response = \
-            '{"results": [{"series": [{"measurement": "sdfsdfsdf", ' \
-            '"columns": ["time", "value"], "values": ' \
-            '[["2009-11-10T23:00:00Z", 0.64]]}]}, {"series": ' \
-            '[{"measurement": "cpu_load_short", "columns": ["time", "value"], ' \
+        example_response = (
+            '{"results": [{"series": [{"measurement": "sdfsdfsdf", '
+            '"columns": ["time", "value"], "values": '
+            '[["2009-11-10T23:00:00Z", 0.64]]}]}, {"series": '
+            '[{"measurement": "cpu_load_short", "columns": ["time", "value"], '
             '"values": [["2009-11-10T23:00:00Z", 0.64]]}]}]}'
+        )
 
         with requests_mock.Mocker() as m:
             m.register_uri(
@@ -405,7 +414,33 @@ class TestInfluxDBClient(unittest.TestCase):
             self.cli.create_database('new_db')
             self.assertEqual(
                 m.last_request.qs['q'][0],
-                'create database new_db'
+                'create database "new_db"'
+            )
+
+    def test_create_database_with_exist_check(self):
+        with requests_mock.Mocker() as m:
+            m.register_uri(
+                requests_mock.GET,
+                "http://localhost:8086/query",
+                text='{"results":[{}]}'
+            )
+            self.cli.create_database('new_db', if_not_exists=True)
+            self.assertEqual(
+                m.last_request.qs['q'][0],
+                'create database if not exists "new_db"'
+            )
+
+    def test_create_numeric_named_database(self):
+        with requests_mock.Mocker() as m:
+            m.register_uri(
+                requests_mock.GET,
+                "http://localhost:8086/query",
+                text='{"results":[{}]}'
+            )
+            self.cli.create_database('123')
+            self.assertEqual(
+                m.last_request.qs['q'][0],
+                'create database "123"'
             )
 
     @raises(Exception)
@@ -423,7 +458,20 @@ class TestInfluxDBClient(unittest.TestCase):
             self.cli.drop_database('new_db')
             self.assertEqual(
                 m.last_request.qs['q'][0],
-                'drop database new_db'
+                'drop database "new_db"'
+            )
+
+    def test_drop_numeric_named_database(self):
+        with requests_mock.Mocker() as m:
+            m.register_uri(
+                requests_mock.GET,
+                "http://localhost:8086/query",
+                text='{"results":[{}]}'
+            )
+            self.cli.drop_database('123')
+            self.assertEqual(
+                m.last_request.qs['q'][0],
+                'drop database "123"'
             )
 
     @raises(Exception)
@@ -454,6 +502,39 @@ class TestInfluxDBClient(unittest.TestCase):
         with _mocked_session(cli, 'get', 401):
             cli.get_list_database()
 
+    def test_get_list_servers(self):
+        data = {'results': [
+            {'series': [
+                {'columns': ['id', 'cluster_addr', 'raft', 'raft-leader'],
+                 'values': [
+                    [1, 'server01:8088', True, True],
+                    [2, 'server02:8088', True, False],
+                    [3, 'server03:8088', True, False]]}]}
+        ]}
+
+        with _mocked_session(self.cli, 'get', 200, json.dumps(data)):
+            self.assertListEqual(
+                self.cli.get_list_servers(),
+                [{'cluster_addr': 'server01:8088',
+                  'id': 1,
+                  'raft': True,
+                  'raft-leader': True},
+                 {'cluster_addr': 'server02:8088',
+                  'id': 2,
+                  'raft': True,
+                  'raft-leader': False},
+                 {'cluster_addr': 'server03:8088',
+                  'id': 3,
+                  'raft': True,
+                  'raft-leader': False}]
+            )
+
+    @raises(Exception)
+    def test_get_list_servers_fails(self):
+        cli = InfluxDBClient('host', 8086, 'username', 'password')
+        with _mocked_session(cli, 'get', 401):
+            cli.get_list_servers()
+
     def test_get_list_series(self):
         example_response = \
             '{"results": [{"series": [{"name": "cpu_load_short", "columns": ' \
@@ -715,7 +796,9 @@ class TestInfluxDBClient(unittest.TestCase):
 
 
 class FakeClient(InfluxDBClient):
-    fail = False
+
+    def __init__(self, *args, **kwargs):
+        super(FakeClient, self).__init__(*args, **kwargs)
 
     def query(self,
               query,
@@ -724,9 +807,10 @@ class FakeClient(InfluxDBClient):
               database=None):
         if query == 'Fail':
             raise Exception("Fail")
-
-        if self.fail:
-            raise Exception("Fail")
+        elif query == 'Fail once' and self._host == 'host1':
+            raise Exception("Fail Once")
+        elif query == 'Fail twice' and self._host in 'host1 host2':
+            raise Exception("Fail Twice")
         else:
             return "Success"
 
@@ -747,32 +831,28 @@ class TestInfluxDBClusterClient(unittest.TestCase):
                                         database='database',
                                         shuffle=False,
                                         client_base_class=FakeClient)
-        self.assertEqual(3, len(cluster.clients))
-        self.assertEqual(0, len(cluster.bad_clients))
-        for idx, client in enumerate(cluster.clients):
-            self.assertEqual(self.hosts[idx][0], client._host)
-            self.assertEqual(self.hosts[idx][1], client._port)
+        self.assertEqual(3, len(cluster.hosts))
+        self.assertEqual(0, len(cluster.bad_hosts))
+        self.assertIn((cluster._client._host,
+                       cluster._client._port), cluster.hosts)
 
     def test_one_server_fails(self):
         cluster = InfluxDBClusterClient(hosts=self.hosts,
                                         database='database',
                                         shuffle=False,
                                         client_base_class=FakeClient)
-        cluster.clients[0].fail = True
-        self.assertEqual('Success', cluster.query(''))
-        self.assertEqual(2, len(cluster.clients))
-        self.assertEqual(1, len(cluster.bad_clients))
+        self.assertEqual('Success', cluster.query('Fail once'))
+        self.assertEqual(2, len(cluster.hosts))
+        self.assertEqual(1, len(cluster.bad_hosts))
 
     def test_two_servers_fail(self):
         cluster = InfluxDBClusterClient(hosts=self.hosts,
                                         database='database',
                                         shuffle=False,
                                         client_base_class=FakeClient)
-        cluster.clients[0].fail = True
-        cluster.clients[1].fail = True
-        self.assertEqual('Success', cluster.query(''))
-        self.assertEqual(1, len(cluster.clients))
-        self.assertEqual(2, len(cluster.bad_clients))
+        self.assertEqual('Success', cluster.query('Fail twice'))
+        self.assertEqual(1, len(cluster.hosts))
+        self.assertEqual(2, len(cluster.bad_hosts))
 
     def test_all_fail(self):
         cluster = InfluxDBClusterClient(hosts=self.hosts,
@@ -781,8 +861,8 @@ class TestInfluxDBClusterClient(unittest.TestCase):
                                         client_base_class=FakeClient)
         with self.assertRaises(InfluxDBServerError):
             cluster.query('Fail')
-        self.assertEqual(0, len(cluster.clients))
-        self.assertEqual(3, len(cluster.bad_clients))
+        self.assertEqual(0, len(cluster.hosts))
+        self.assertEqual(3, len(cluster.bad_hosts))
 
     def test_all_good(self):
         cluster = InfluxDBClusterClient(hosts=self.hosts,
@@ -790,8 +870,8 @@ class TestInfluxDBClusterClient(unittest.TestCase):
                                         shuffle=True,
                                         client_base_class=FakeClient)
         self.assertEqual('Success', cluster.query(''))
-        self.assertEqual(3, len(cluster.clients))
-        self.assertEqual(0, len(cluster.bad_clients))
+        self.assertEqual(3, len(cluster.hosts))
+        self.assertEqual(0, len(cluster.bad_hosts))
 
     def test_recovery(self):
         cluster = InfluxDBClusterClient(hosts=self.hosts,
@@ -801,68 +881,57 @@ class TestInfluxDBClusterClient(unittest.TestCase):
         with self.assertRaises(InfluxDBServerError):
             cluster.query('Fail')
         self.assertEqual('Success', cluster.query(''))
-        self.assertEqual(1, len(cluster.clients))
-        self.assertEqual(2, len(cluster.bad_clients))
+        self.assertEqual(1, len(cluster.hosts))
+        self.assertEqual(2, len(cluster.bad_hosts))
+
+    def test_healing(self):
+        cluster = InfluxDBClusterClient(hosts=self.hosts,
+                                        database='database',
+                                        shuffle=True,
+                                        healing_delay=1,
+                                        client_base_class=FakeClient)
+        with self.assertRaises(InfluxDBServerError):
+            cluster.query('Fail')
+        self.assertEqual('Success', cluster.query(''))
+        time.sleep(1.1)
+        self.assertEqual('Success', cluster.query(''))
+        self.assertEqual(2, len(cluster.hosts))
+        self.assertEqual(1, len(cluster.bad_hosts))
+        time.sleep(1.1)
+        self.assertEqual('Success', cluster.query(''))
+        self.assertEqual(3, len(cluster.hosts))
+        self.assertEqual(0, len(cluster.bad_hosts))
 
     def test_dsn(self):
         cli = InfluxDBClusterClient.from_DSN(self.dsn_string)
-        self.assertEqual(2, len(cli.clients))
-        self.assertEqual('http://host1:8086', cli.clients[0]._baseurl)
-        self.assertEqual('uSr', cli.clients[0]._username)
-        self.assertEqual('pWd', cli.clients[0]._password)
-        self.assertEqual('db', cli.clients[0]._database)
-        self.assertFalse(cli.clients[0].use_udp)
-        self.assertEqual('http://host2:8086', cli.clients[1]._baseurl)
-        self.assertEqual('uSr', cli.clients[1]._username)
-        self.assertEqual('pWd', cli.clients[1]._password)
-        self.assertEqual('db', cli.clients[1]._database)
-        self.assertFalse(cli.clients[1].use_udp)
+        self.assertEqual([('host1', 8086), ('host2', 8086)], cli.hosts)
+        self.assertEqual('http://host1:8086', cli._client._baseurl)
+        self.assertEqual('uSr', cli._client._username)
+        self.assertEqual('pWd', cli._client._password)
+        self.assertEqual('db', cli._client._database)
+        self.assertFalse(cli._client.use_udp)
 
         cli = InfluxDBClusterClient.from_DSN('udp+' + self.dsn_string)
-        self.assertTrue(cli.clients[0].use_udp)
-        self.assertTrue(cli.clients[1].use_udp)
+        self.assertTrue(cli._client.use_udp)
 
         cli = InfluxDBClusterClient.from_DSN('https+' + self.dsn_string)
-        self.assertEqual('https://host1:8086', cli.clients[0]._baseurl)
-        self.assertEqual('https://host2:8086', cli.clients[1]._baseurl)
+        self.assertEqual('https://host1:8086', cli._client._baseurl)
 
         cli = InfluxDBClusterClient.from_DSN('https+' + self.dsn_string,
                                              **{'ssl': False})
-        self.assertEqual('http://host1:8086', cli.clients[0]._baseurl)
-        self.assertEqual('http://host2:8086', cli.clients[1]._baseurl)
-
-    def test_dsn_single_client(self):
-        cli = InfluxDBClusterClient.from_DSN('influxdb://usr:pwd@host:8086/db')
-        self.assertEqual('http://host:8086', cli.clients[0]._baseurl)
-        self.assertEqual('usr', cli.clients[0]._username)
-        self.assertEqual('pwd', cli.clients[0]._password)
-        self.assertEqual('db', cli.clients[0]._database)
-        self.assertFalse(cli.clients[0].use_udp)
-
-        cli = InfluxDBClusterClient.from_DSN(
-            'udp+influxdb://usr:pwd@host:8086/db')
-        self.assertTrue(cli.clients[0].use_udp)
-
-        cli = InfluxDBClusterClient.from_DSN(
-            'https+influxdb://usr:pwd@host:8086/db')
-        self.assertEqual('https://host:8086', cli.clients[0]._baseurl)
-
-        cli = InfluxDBClusterClient.from_DSN(
-            'https+influxdb://usr:pwd@host:8086/db',
-            **{'ssl': False})
-        self.assertEqual('http://host:8086', cli.clients[0]._baseurl)
+        self.assertEqual('http://host1:8086', cli._client._baseurl)
 
     def test_dsn_password_caps(self):
         cli = InfluxDBClusterClient.from_DSN(
             'https+influxdb://usr:pWd@host:8086/db')
-        self.assertEqual('pWd', cli.clients[0]._password)
+        self.assertEqual('pWd', cli._client._password)
 
     def test_dsn_mixed_scheme_case(self):
         cli = InfluxDBClusterClient.from_DSN(
             'hTTps+inFLUxdb://usr:pWd@host:8086/db')
-        self.assertEqual('pWd', cli.clients[0]._password)
-        self.assertEqual('https://host:8086', cli.clients[0]._baseurl)
+        self.assertEqual('pWd', cli._client._password)
+        self.assertEqual('https://host:8086', cli._client._baseurl)
 
         cli = InfluxDBClusterClient.from_DSN(
             'uDP+influxdb://usr:pwd@host1:8086,usr:pwd@host2:8086/db')
-        self.assertTrue(cli.clients[0].use_udp)
+        self.assertTrue(cli._client.use_udp)
