@@ -15,7 +15,11 @@ from __future__ import print_function
 from functools import partial
 import os
 import time
-import unittest
+import sys
+if sys.version_info < (2, 7):
+    import unittest2 as unittest
+else:
+    import unittest
 import warnings
 
 # By default, raise exceptions on warnings
@@ -131,7 +135,12 @@ class SimpleTests(SingleTestCaseWithServerMixin,
             [{'name': 'new_db_1'}, {'name': 'new_db_2'}]
         )
 
-    def test_create_database_fails(self):
+    def test_create_database_twice_if_not_exist(self):
+        self.assertIsNone(self.cli.create_database('new_db'))
+        self.assertIsNone(
+            self.cli.create_database('new_db', if_not_exists=True))
+
+    def test_create_database_twice_fails(self):
         self.assertIsNone(self.cli.create_database('new_db'))
         with self.assertRaises(InfluxDBClientError) as ctx:
             self.cli.create_database('new_db')
@@ -303,8 +312,8 @@ class CommonTests(ManyTestCasesWithServerMixin,
         self.test_write()
         time.sleep(1)
         rsp = self.cli.query('SELECT * FROM cpu_load_short', database='db')
-        self.assertListEqual([{'value': 0.64,
-                               'time': '2009-11-10T23:00:00Z'}],
+        self.assertListEqual([{'value': 0.64, 'time': '2009-11-10T23:00:00Z',
+                               "host": "server01", "region": "us-west"}],
                              list(rsp.get_points()))
 
     def test_write_points(self):
@@ -328,7 +337,8 @@ class CommonTests(ManyTestCasesWithServerMixin,
 
         self.assertEqual(
             list(rsp),
-            [[{'value': 0.64, 'time': '2009-11-10T23:00:00Z'}]]
+            [[{'value': 0.64, 'time': '2009-11-10T23:00:00Z',
+               "host": "server01", "region": "us-west"}]]
         )
 
         rsp2 = list(rsp.get_points())
@@ -337,7 +347,8 @@ class CommonTests(ManyTestCasesWithServerMixin,
 
         self.assertEqual(
             pt,
-            {'time': '2009-11-10T23:00:00Z', 'value': 0.64}
+            {'time': '2009-11-10T23:00:00Z', 'value': 0.64,
+             "host": "server01", "region": "us-west"}
         )
 
     @unittest.skip("Broken as of 0.9.0")
@@ -367,7 +378,8 @@ class CommonTests(ManyTestCasesWithServerMixin,
         lrsp = list(rsp)
 
         self.assertEqual(
-            [[{'value': 0.64, 'time': '2009-11-10T23:00:00Z'}]],
+            [[{'value': 0.64, 'time': '2009-11-10T23:00:00Z',
+               "host": "server01", "region": "us-west"}]],
             lrsp
         )
 
@@ -375,7 +387,8 @@ class CommonTests(ManyTestCasesWithServerMixin,
 
         self.assertEqual(
             rsp,
-            [[{'value': 33, 'time': '2009-11-10T23:01:35Z'}]]
+            [[{'value': 33, 'time': '2009-11-10T23:01:35Z',
+               "host": "server01", "region": "us-west"}]]
         )
 
     @unittest.skip("Broken as of 0.9.0")
@@ -678,6 +691,7 @@ class UdpTests(ManyTestCasesWithServerMixin,
 
         self.assertEqual(
             # this is dummy_points :
-            [{'value': 0.64, 'time': '2009-11-10T23:00:00Z'}],
+            [{'value': 0.64, 'time': '2009-11-10T23:00:00Z',
+              "host": "server01", "region": "us-west"}],
             list(rsp['cpu_load_short'])
         )
