@@ -1,6 +1,11 @@
 # -*- coding: utf-8 -*-
 
-import unittest
+import sys
+if sys.version_info < (2, 7):
+    import unittest2 as unittest
+else:
+    import unittest
+
 from influxdb import line_protocol
 
 
@@ -20,7 +25,9 @@ class TestLineProtocol(unittest.TestCase):
                     "fields": {
                         "string_val": "hello!",
                         "int_val": 1,
+                        "float_val": 1.1,
                         "none_field": None,
+                        "bool_val": True,
                     }
                 }
             ]
@@ -29,7 +36,7 @@ class TestLineProtocol(unittest.TestCase):
         self.assertEqual(
             line_protocol.make_lines(data),
             'test,integer_tag=2,string_tag=hello '
-            'int_val=1,string_val="hello!"\n'
+            'bool_val=True,float_val=1.1,int_val=1i,string_val="hello!"\n'
         )
 
     def test_string_val_newline(self):
